import {Component} from '@angular/core';
import {MessageService} from "primeng/api";
import {Router} from "@angular/router";
import {ScannerService} from "../scanner.service";
import {LoaderService} from "../../../core/services/loader.service";

@Component({
  selector: 'app-scanner',
  templateUrl: './scanner.component.html'
})
export class ScannerComponent {

  constructor(private messageService: MessageService,
              private loaderService: LoaderService,
              private service: ScannerService,
              private router: Router) {
  }

  get originalFileValidated(): boolean | null {
    return this.service.originalFileValidated;
  }

  set originalFileValidated(value: boolean | null) {
    this.service.originalFileValidated = value;
  }

  get scannedFileValidated(): boolean | null {
    return this.service.scannedFileValidated;
  }

  set scannedFileValidated(value: boolean | null) {
    this.service.scannedFileValidated = value;
  }

  get originalFile(): File | null {
    return this.service.originalFile;
  }

  set originalFile(value: File | null) {
    this.service.originalFile = value;
  }

  get scannedFile(): File | null {
    return this.service.scannedFile;
  }

  set scannedFile(value: File | null) {
    this.service.scannedFile = value;
  }

  get originalUrl(): string {
    return this.service.originalUrl;
  }

  set originalUrl(value: string) {
    this.service.originalUrl = value;
  }

  get scannedUrl(): string {
    return this.service.scannedUrl;
  }

  set scannedUrl(value: string) {
    this.service.scannedUrl = value;
  }

  onFileSelected(event: any, type: string): void {
    const file: File = event.target.files[0];
    //accept only pdf files
    if (file && file.type === 'application/pdf') {
      this.handleFile(file, type)
    } else {
      this.messageService.add({
        severity: 'warn',
        summary: 'warning',
        detail: 'Only pdf file allowed.'
      });
    }
  }

  onFileDropped(event: DragEvent, type: string): void {
    event.preventDefault();
    const file = event.dataTransfer?.files[0];
    //accept only pdf files
    if (file && file.type === 'application/pdf') {
      this.handleFile(file, type)
    } else {
      this.messageService.add({
        severity: 'warn',
        summary: 'warning',
        detail: 'Only pdf file allowed.'
      });
    }
  }

  handleFile(file: File, type: string) {

    if (type === 'ORIGINAL') {
      this.originalFileValidated = true
      this.originalFile = file
    }

    if (type === 'SCANNED') {
      this.scannedFileValidated = true
      this.scannedFile = file
    }

  }

  onDragOver(event: DragEvent): void {
    event.preventDefault();
  }


  startScan() {
    // check if the original file is not empty
    if (this.originalFile === null) {
      this.originalFileValidated = false
      return
    }

    // check if the scanned file is not empty
    if (this.scannedFile === null) {
      this.scannedFileValidated = false
      return
    }

    //show loader
    this.loaderService.show()
    //scan the two files
    this.service.scanFile(this.originalFile, this.scannedFile)
      .subscribe(res => {
        console.log(res)
        this.loaderService.hide()
        if (res?.statuscode === '000') {
          localStorage.setItem('score', res.Score)
          this.router.navigate(['scan-completed/' + res?.original_pdf?.filename + '/' + res?.scanned_pdf?.filename])
        } else {
          this.loaderService.hide()
          this.messageService.add({
            severity: 'error',
            summary: 'Error',
            detail: res?.message
          })
        }
      }, error => {
        this.loaderService.hide()
        console.log(error)
        this.messageService.add({
          severity: 'error',
          summary: 'Error',
          detail: error?.error?.message || 'something went wrong.'
        })
      })
  }

  /**
   * takes as parameter Blob and generate url for the file
   * @param pdfBlob
   */
  savePDF(pdfBlob: Blob) {
    const file = new Blob([pdfBlob], {type: 'application/pdf'});
    const fileURL = URL.createObjectURL(file);
    this.originalUrl = fileURL
    this.scannedUrl = fileURL
    this.router.navigate(['/scanner-completed/view'])
  }
}
